package ch.ethz.se.rpnc;

import java.util.Observable;
import java.util.Observer;

import ch.ethz.se.rpnc.instruct.CalculatorCommandType;
import ch.ethz.se.rpnc.instruct.CalculatorInstructCommand;
import ch.ethz.se.rpnc.instruct.CalculatorInstructOperand;
import ch.ethz.se.rpnc.instruct.CalculatorInstructOperator;
import ch.ethz.se.rpnc.instruct.CalculatorOperatorType;
import ch.ethz.se.rpnc.instruct.ICalculatorInstruct;
import ch.ethz.se.rpnc.instruct.visitor.CalculatorInstructVisitorAdapter;
import ch.ethz.se.rpnc.ui.CalculatorInstructEvent;
import ch.ethz.se.rpnc.ui.ICalculatorView;

public class CalculatorController implements Observer {

	class CalculatorInstructControllerExecutor extends
			CalculatorInstructVisitorAdapter {
		
		// Is the instruct considered processed after the visit?
		private boolean isInstructProcessed;

		/**
		 * Indicate whether the instruct has been processed.
		 * 
		 * @param isInstructProcessed
		 *            A flag.
		 */
		public void setInstructProcessed(boolean isInstructProcessed) {
			this.isInstructProcessed = isInstructProcessed;
		}

		/**
		 * Query for if the instruct has been processed.
		 * 
		 * @return A flag.
		 */
		public boolean isInstructProcessed() {
			return isInstructProcessed;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * ch.ethz.se.rpnc.instruct.visitor.CalculatorInstructVisitorAdapter
		 * #visitCommand(ch.ethz.se.rpnc.instruct.CalculatorInstructCommand)
		 */
		@Override
		public void visitCommand(CalculatorInstructCommand instruct)
				throws Exception {
			setInstructProcessed(false);
			if (instruct == CalculatorInstructCommand
					.getInstructCommand(CalculatorCommandType.CMD_CLEAR)) {
				calculator.clearStack();
				setInstructProcessed(true);
			} else if (instruct == CalculatorInstructCommand
					.getInstructCommand(CalculatorCommandType.CMD_EVALUATE)) {
				calculator.evaluateStack();
				setInstructProcessed(true);
			}
			assert (isInstructProcessed());
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * ch.ethz.se.rpnc.instruct.visitor.CalculatorInstructVisitorAdapter
		 * #visitOperand(ch.ethz.se.rpnc.instruct.CalculatorInstructOperand)
		 */
		@Override
		public void visitOperand(CalculatorInstructOperand instruct) {
			calculator.pushOperand(instruct.getValue());
			setInstructProcessed(true);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * ch.ethz.se.rpnc.instruct.visitor.CalculatorInstructVisitorAdapter
		 * #visitOperator(ch.ethz.se.rpnc.instruct.CalculatorInstructOperator)
		 */
		@Override
		public void visitOperator(CalculatorInstructOperator instruct) {
			if (instruct == CalculatorInstructOperator
					.getInstructOperator(CalculatorOperatorType.OPERATOR_PLUS)) {
				calculator.pushPlusOperator();
			} else if (instruct == CalculatorInstructOperator
					.getInstructOperator(CalculatorOperatorType.OPERATOR_MINUS)) {
				calculator.pushMinusOperator();
			} else if (instruct == CalculatorInstructOperator
					.getInstructOperator(CalculatorOperatorType.OPERATOR_TIMES)) {
				calculator.pushTimesOperator();
			} else if (instruct == CalculatorInstructOperator
					.getInstructOperator(CalculatorOperatorType.OPERATOR_DIVIDEBY)) {
				calculator.pushDivideByOperator();
			}
			setInstructProcessed(true);
		}

	}

	// Calculator as the model of MVC pattern.
	private ReversePolishNotationCalculator calculator;

	// Console as the view of MVC pattern.
	private ICalculatorView view;

	// Instruct controller executor.
	private CalculatorInstructControllerExecutor executor;

	/**
	 * Construct a controller, associating the model and view.
	 * 
	 * @param aCalc
	 *            A calculator.
	 * @param aConsole
	 *            A console.
	 */
	public CalculatorController(ReversePolishNotationCalculator aCalc,
			ICalculatorView aView) {
		calculator = aCalc;

		// Inform the controller of any change in console command.
		view = aView;
		view.addCommandObserver(this);
		executor = new CalculatorInstructControllerExecutor();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	@Override
	public void update(Observable o, Object arg) {
		CalculatorInstructEvent input = (CalculatorInstructEvent) o;
		ICalculatorInstruct instruct = input.getInstruct();
		assert (instruct != null);

		try {
			instruct.accept(executor);
		} catch (Exception e) {
			view.printMessage(e.getMessage());
		}
	}

}
