package ch.ethz.se.rpnc;

import java.util.Stack;

import ch.ethz.se.rpnc.exception.EmptyStackEvaluationException;
import ch.ethz.se.rpnc.node.DivideByOperator;
import ch.ethz.se.rpnc.node.MinusOperator;
import ch.ethz.se.rpnc.node.Node;
import ch.ethz.se.rpnc.node.Operand;
import ch.ethz.se.rpnc.node.PlusOperator;
import ch.ethz.se.rpnc.node.TimesOperator;
import ch.ethz.se.rpnc.node.visitor.Evaluator;
import ch.ethz.se.rpnc.node.visitor.NodePrinter;
import ch.ethz.se.rpnc.ui.ResourceManager;
import ch.ethz.se.rpnc.ui.UIMessage;

/**
 * Calculator based on the reverse polish notation.
 */
public class ReversePolishNotationCalculator {

	/**
	 * Run Calculator Demo.
	 * 
	 * @param args
	 *            Command line arguments
	 */
	public static void main(String[] args) {
		ReversePolishNotationCalculator rpnc = new ReversePolishNotationCalculator();
		rpnc.pushOperand(5);
		rpnc.pushOperand(4);
		rpnc.pushOperand(2);
		rpnc.pushPlusOperator();
		try {
			rpnc.evaluateStack();
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
	}

	/**
	 * Stack of current operands
	 */
	private Stack<Node> stack;

	/**
	 * Operation evaluator
	 */
	private Evaluator evaluator;

	/**
	 * Create new calculator.
	 */
	public ReversePolishNotationCalculator() {
		stack = new Stack<Node>();
		evaluator = new Evaluator(stack);
	}

	/**
	 * Get the stack of caclulator.
	 * 
	 * @return stack
	 */
	public Stack<Node> getStack() {
		return stack;
	}

	/**
	 * test for emptyness of the stack.
	 * 
	 * @return true if the stack is empty, false otherwise
	 */
	public boolean isStackEmpty() {
		return stack.isEmpty();
	}

	/**
	 * get the topmost operand as a double.
	 * 
	 * @return the value of the operand
	 * @throws Exception
	 *             if the topmost element is not an operand
	 */
	public double popOperand() throws Exception {
		return ((Operand) stack.pop()).getValue();
	}

	/**
	 * peek the value of the topmost operand, but leave it on the stack.
	 * 
	 * @return the value of the operand
	 * @throws Exception
	 *             if the topmost element is not an operand
	 */
	public double peekOperand() throws Exception {
		return ((Operand) stack.peek()).getValue();
	}

	/**
	 * Removes all elements from the stack.
	 */
	public void clearStack() {
		stack.clear();
	}

	/**
	 * Push Plus operator on top of stack.
	 */
	public void pushPlusOperator() {
		stack.add(new PlusOperator());
	}

	/**
	 * Push Minus operator on top of stack.
	 */
	public void pushMinusOperator() {
		stack.add(new MinusOperator());
	}

	/**
	 * Push Times operator on top of stack.
	 */
	public void pushTimesOperator() {
		stack.add(new TimesOperator());
	}

	/**
	 * Push DivideBy operator on top of stack.
	 */
	public void pushDivideByOperator() {
		stack.add(new DivideByOperator());
	}

	/**
	 * Push number on top of stack.
	 * 
	 * @param o
	 *            Number to push on stack
	 */
	public void pushOperand(double o) {
		stack.add(new Operand(o));
	}

	/**
	 * Evaluate stack.
	 */
	public void evaluateStack() throws Exception {
		if (stack.empty()) {
			throw new EmptyStackEvaluationException(
					ResourceManager
							.getLocalizedUIMessage(UIMessage.MSG_MEMORY_EMPTY));
		}
		Node n = stack.pop();
		assert (n != null);
		n.accept(evaluator);
		stack.push(new Operand(evaluator.getResult()));
	}

	/**
	 * To String representation.
	 */
	public String toString() {
		NodePrinter printer=new NodePrinter();
		printer.reset();
		try {
			for (Node n : stack) {
				n.accept(printer);
			}
		} catch (Exception e) {
			// Should not happen.
			assert(false);
		}
		return printer.getPrintResult().trim();
	}
}
